@@ -166,9 +166,11 @@ def is_iscsi_device(dev, node_uuid):
     return iscsi_id in dev
 
 
-def is_nvme_device(dev):
-    """check whether the device path belongs to an NVMe drive. """
-    return "/dev/nvme" in dev
+def is_last_char_digit(dev):
+    """check whether device name ends with a digit"""
+    if len(dev) >= 1:
+        return dev[-1].isdigit()
+    return False
 
 
 def make_partitions(dev, root_mb, swap_mb, ephemeral_mb,
@@ -216,7 +218,7 @@ def make_partitions(dev, root_mb, swap_mb, ephemeral_mb,
     # the device partitions as /dev/sda1 and not /dev/sda-part1.
     if is_iscsi_device(dev, node_uuid):
         part_template = dev + '-part%d'
-    elif is_nvme_device(dev):
+    elif is_last_char_digit(dev):
         part_template = dev + 'p%d'
     else:
         part_template = dev + '%d'
@@ -292,6 +294,7 @@ def make_partitions(dev, root_mb, swap_mb, ephemeral_mb,
     if commit:
         # write to the disk
         dp.commit()
+        _trigger_device_rescan(dev)
     return part_dict
 
 
@@ -414,7 +417,20 @@ def destroy_disk_metadata(dev, node_uuid):
                 utils.execute('wipefs', '--all', dev,
                               run_as_root=True,
                               use_standard_locale=True)
-
+    # NOTE(TheJulia): sgdisk attempts to load and make sense of the
+    # partition tables in advance of wiping the partition data.
+    # This means when a CRC error is found, sgdisk fails before
+    # erasing partition data.
+    # This is the same bug as
+    # https://bugs.launchpad.net/ironic-python-agent/+bug/1737556
+    dd_device = 'of=%s' % dev
+    gpt_backup = get_dev_block_size(dev) - 33
+    dd_seek = 'seek=%i' % gpt_backup
+    utils.execute('dd', 'bs=512', 'if=/dev/zero', dd_device, 'count=33',
+                  run_as_root=True, use_standard_locale=True)
+    utils.execute('dd', 'bs=512', 'if=/dev/zero', dd_device, 'count=33',
+                  dd_seek, run_as_root=True, use_standard_locale=True)
+    # Go ahead and let sgdisk run as well.
     utils.execute('sgdisk', '-Z', dev, run_as_root=True,
                   use_standard_locale=True)
 
@@ -794,6 +810,45 @@ def fix_gpt_partition(device, node_uuid):
         raise exception.InstanceDeployFailure(msg)
 
 
+def _trigger_device_rescan(device):
+    """Sync and trigger device rescan.
+
+    Disk partition performed via parted, when performed on a ramdisk
+    do not have to honor the fsync mechanism. In essence, fsync is used
+    on the file representing the block device, which falls to the kernel
+    filesystem layer to trigger a sync event. On a ramdisk using ramfs,
+    this is an explicit non-operation.
+
+    As a result of this, we need to trigger a system wide sync operation
+    which will trigger cache to flush to disk, after which partition changes
+    should be visible upon re-scan.
+
+    When ramdisks are not in use, this also helps ensure that data has
+    been safely flushed across the wire, such as on iscsi connections.
+
+    :param device: The block device containing paritions that is attempting
+                   to be updated.
+    """
+    # TODO(TheJulia): This helper method was broken out for re-use, and
+    # does not have an explicit test case for itself, and it should, but
+    # it's steps are fairly explicitly checked with mocks on execute in
+    # the partition code.
+    LOG.debug('Explicitly calling sync to force buffer/cache flush.')
+    utils.execute('sync')
+    # Make sure any additions to the partitioning are reflected in the
+    # kernel.
+    LOG.debug('Waiting until udev event queue is empty')
+    utils.execute('udevadm', 'settle')
+    try:
+        utils.execute('partprobe', device, run_as_root=True,
+                      attempts=CONF.disk_utils.partprobe_attempts)
+        # Also verify that the partitioning is correct now.
+        utils.execute('sgdisk', '-v', device, run_as_root=True)
+    except processutils.ProcessExecutionError as exc:
+        LOG.warning('Failed to verify partitioning after creating '
+                    'partition(s): %s', exc)
+
+
 def create_config_drive_partition(node_uuid, device, configdrive):
     """Create a partition for config drive
 
@@ -876,25 +931,8 @@ def create_config_drive_partition(node_uuid, device, configdrive):
                 utils.execute('parted', '-a', 'optimal', '-s', '--', device,
                               'mkpart', 'primary', 'fat32', startlimit,
                               endlimit, run_as_root=True)
-            # Parted uses fsync to tell the kernel to sync file io
-            # however on ramdisks in ramfs, this is an explicit no-op.
-            # Explicitly call sync so when the the kernel attempts to read
-            # the partition table from disk, it is less likely that the write
-            # is still in buffer cache pending write to disk.
-            LOG.debug('Explicitly calling sync to force buffer/cache flush.')
-            utils.execute('sync')
-            # Make sure any additions to the partitioning are reflected in the
-            # kernel.
-            LOG.debug('Waiting until udev event queue is empty')
-            utils.execute('udevadm', 'settle')
-            try:
-                utils.execute('partprobe', device, run_as_root=True,
-                              attempts=CONF.disk_utils.partprobe_attempts)
-                # Also verify that the partitioning is correct now.
-                utils.execute('sgdisk', '-v', device, run_as_root=True)
-            except processutils.ProcessExecutionError as exc:
-                LOG.warning('Failed to verify GPT partitioning after creating '
-                            'the configdrive partition: %s', exc)
+            # Trigger device rescan
+            _trigger_device_rescan(device)
 
             upd_parts = set(part['number'] for part in list_partitions(device))
             new_part = set(upd_parts) - set(cur_parts)
@@ -906,7 +944,7 @@ def create_config_drive_partition(node_uuid, device, configdrive):
 
             if is_iscsi_device(device, node_uuid):
                 config_drive_part = '%s-part%s' % (device, new_part.pop())
-            elif is_nvme_device(device):
+            elif is_last_char_digit(device):
                 config_drive_part = '%sp%s' % (device, new_part.pop())
             else:
                 config_drive_part = '%s%s' % (device, new_part.pop())
